"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parsePatch = parsePatch;
exports.parseMultiPatch = parseMultiPatch;
exports.parseUnifiedDiff = parseUnifiedDiff;

function parsePatch(contents) {
  var patch = {
    sha: /^From ([^ ]+)/.exec(contents)[1],
    message: /^Subject: \[.+?\] ([\S\s]+)?^---$/m.exec(contents)[1].trim(),
    changes: []
  }; // Remove the weird -- 2.2.1 part at the end of every patch

  contents = contents.split(/^-- $/m)[0];
  var fileChunks = contents.split(/^diff --git /m);
  fileChunks.slice(1).forEach(function (chunk) {
    // Parse the old filename.
    var fileMatch = /^--- a\/(.+)$/m.exec(chunk);
    var file = fileMatch && fileMatch[1]; // Parse the new filename.

    var destMatch = /^\+\+\+ b\/(.+)$/m.exec(chunk);
    var dest = destMatch && destMatch[1];

    if (!file && !dest) {
      var renameFrom = /^rename from (.+)$/m.exec(chunk);
      var renameTo = /^rename to (.+)$/m.exec(chunk);

      if (renameFrom && renameTo) {
        file = renameFrom[1];
        dest = renameTo[1];
      }
    }

    var change = {
      type: file ? dest ? "change" : "delete" : "add",
      file: file || dest
    };
    var diff = chunk.slice(chunk.indexOf("@@"));

    if (file) {
      change.diff = parseUnifiedDiff(diff);

      if (change.diff.length) {
        patch.changes.push(change);
      }

      if (dest && file !== dest) {
        patch.changes.push({
          type: "rename",
          file: file,
          dest: dest
        });
      }
    } else {
      change.text = parseFirstChunk(diff);
      patch.changes.push(change);
    }
  });
  return patch;
}

function parseMultiPatch(contents) {
  var patchStart = /^From /gm;
  var match = null;
  var patchIndices = [];

  while ((match = patchStart.exec(contents)) != null) {
    patchIndices.push(match.index);
  }

  return patchIndices.map(function (_, i) {
    return parsePatch(contents.slice(patchIndices[i], patchIndices[i + 1]));
  });
}

var lineRangeRE = /^@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@/;

function parseUnifiedDiff(contents) {
  var chunks = []; // Ignore the last line which is always empty

  var lines = contents.split(/\r?\n/).slice(0, -1);

  for (var i = 0; i < lines.length;) {
    var range = lines[i]; // Every chunk begins with two line ranges.

    var rangeMatch = lineRangeRE.exec(range);

    if (!rangeMatch) {
      break;
    }

    var match = rangeMatch.map(Number);
    var chunk = {
      lines: [],
      inputRange: {
        start: match[1],
        length: match[2]
      },
      outputRange: {
        start: match[3],
        length: match[4]
      }
    };

    for (var line = lines[++i]; line && line[0] != "@"; line = lines[++i]) {
      // Lines that begin with a backslash are not actual lines.
      // For example, the removal of a trailing newline.
      if (line[0] == "\\") {
        continue;
      }

      chunk.lines.push({
        prefix: line[0],
        text: line.slice(1)
      });
    }

    chunks.push(chunk);
  }

  return chunks;
} // Parse the contents of the first chunk in a diff.


function parseFirstChunk(diff) {
  return diff.split(/\r?\n/).slice(1).map(function (line) {
    return line.slice(1);
  }).join("\n");
}